@@ -15,6 +15,7 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include <sys/queue.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
 
@@ -61,7 +62,7 @@ struct got_blame {
 	 * These change with every traversed commit. After diffing
 	 * commits N:N-1, in preparation for diffing commits N-1:N-2,
 	 * data for commit N is retained and flipped into data for N-1.
-	 * 
+	 *
 	 */
 	FILE *f1; /* older version from commit N-1. */
 	FILE *f2; /* newer version from commit N. */
@@ -472,7 +473,7 @@ flip_files(struct got_blame *blame)
 
 	err = got_opentemp_truncate(blame->f2);
 	if (err)
-		return err; 
+		return err;
 	tmp = blame->f2;
 	blame->f2 = blame->f1;
 	blame->f1 = tmp;
@@ -502,8 +503,8 @@ blame_open(struct got_blame **blamep, const char *path,
 	struct got_object_id *obj_id = NULL;
 	struct got_blob_object *blob = NULL;
 	struct got_blame *blame = NULL;
-	struct got_object_id *id = NULL;
-	int lineno;
+	struct got_object_id id;
+	int lineno, have_id = 0;
 	struct got_commit_graph *graph = NULL;
 
 	*blamep = NULL;
@@ -583,8 +584,7 @@ blame_open(struct got_blame **blamep, const char *path,
 	if (err)
 		goto done;
 	for (;;) {
-		struct got_object_id *next_id;
-		err = got_commit_graph_iter_next(&next_id, graph, repo,
+		err = got_commit_graph_iter_next(&id, graph, repo,
 		    cancel_cb, cancel_arg);
 		if (err) {
 			if (err->code == GOT_ERR_ITER_COMPLETED) {
@@ -593,30 +593,29 @@ blame_open(struct got_blame **blamep, const char *path,
 			}
 			goto done;
 		}
-		if (next_id) {
-			id = next_id;
-			err = blame_commit(blame, id, path, repo, cb, arg);
-			if (err) {
-				if (err->code == GOT_ERR_ITER_COMPLETED)
-					err = NULL;
-				goto done;
-			}
-			if (blame->nannotated == blame->nlines)
-				break;
+		have_id = 1;
 
-			err = flip_files(blame);
-			if (err)
-				goto done;
+		err = blame_commit(blame, &id, path, repo, cb, arg);
+		if (err) {
+			if (err->code == GOT_ERR_ITER_COMPLETED)
+				err = NULL;
+			goto done;
 		}
+		if (blame->nannotated == blame->nlines)
+			break;
+
+		err = flip_files(blame);
+		if (err)
+			goto done;
 	}
 
-	if (id && blame->nannotated < blame->nlines) {
+	if (have_id && blame->nannotated < blame->nlines) {
 		/* Annotate remaining non-annotated lines with last commit. */
-		err = got_object_open_as_commit(&last_commit, repo, id);
+		err = got_object_open_as_commit(&last_commit, repo, &id);
 		if (err)
 			goto done;
 		for (lineno = 0; lineno < blame->nlines; lineno++) {
-			err = annotate_line(blame, lineno, last_commit, id,
+			err = annotate_line(blame, lineno, last_commit, &id,
 			    cb, arg);
 			if (err)
 				goto done;
