@@ -65,6 +65,7 @@
  */
 
 #include <sys/stat.h>
+#include <sys/queue.h>
 
 #include <ctype.h>
 #include <limits.h>
@@ -239,7 +240,7 @@ diffreg(BUF **d, const char *path1, const char *path2,
 	}
 
 	err = got_diffreg_output(NULL, NULL, diffreg_result, 1, 1, "", "",
-	    GOT_DIFF_OUTPUT_EDSCRIPT, 0, outfile);
+	    GOT_DIFF_OUTPUT_PLAIN, 0, outfile);
 	if (err)
 		goto done;
 
