@@ -14,6 +14,7 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include <sys/queue.h>
 #include <sys/stat.h>
 
 #include <errno.h>
@@ -1202,7 +1203,7 @@ got_fileindex_diff_dir(struct got_fileindex *fileindex, int fd,
 
 	TAILQ_INIT(&dirlist);
 
-	/* 
+	/*
 	 * Duplicate the file descriptor so we can call closedir() below
 	 * without closing the file descriptor passed in by our caller.
 	 */
