@@ -16,6 +16,7 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/queue.h>
 #include <sys/wait.h>
 
 #include <ctype.h>
@@ -272,7 +273,7 @@ te_mode2str(char *buf, size_t len, struct got_tree_entry *te)
 		return got_error(GOT_ERR_BAD_FILETYPE);
 
 	ret = snprintf(buf, len, "%o ", mode);
-	if (ret == -1 || ret >= len)
+	if (ret < 0 || (size_t)ret >= len)
 		return got_error(GOT_ERR_NO_SPACE);
 	return NULL;
 }
@@ -454,8 +455,10 @@ got_object_commit_create(struct got_object_id **id,
 	}
 
 	if (asprintf(&author_str, "%s%s %lld +0000\n",
-	    GOT_COMMIT_LABEL_AUTHOR, author, (long long)author_time) == -1)
-		return got_error_from_errno("asprintf");
+	    GOT_COMMIT_LABEL_AUTHOR, author, (long long)author_time) == -1) {
+		err = got_error_from_errno("asprintf");
+		goto done;
+	}
 
 	if (asprintf(&committer_str, "%s%s %lld +0000\n",
 	    GOT_COMMIT_LABEL_COMMITTER, committer ? committer : author,
@@ -746,10 +749,7 @@ got_object_tag_create(struct got_object_id **id,
 		err = buf_load_fd(&buf, out_fd);
 		if (err)
 			goto done;
-		sig_len = buf_len(buf) + 1;
-		err = buf_putc(buf, '\0');
-		if (err)
-			goto done;
+		sig_len = buf_len(buf);
 		if (close(out_fd) == -1) {
 			err = got_error_from_errno("close");
 			goto done;
