@@ -48,7 +48,6 @@
 #include "got_lib_object_parse.h"
 #include "got_lib_object_cache.h"
 #include "got_lib_pack.h"
-#include "got_lib_privsep.h"
 #include "got_lib_repository.h"
 
 #ifndef nitems
@@ -134,6 +133,9 @@ got_object_raw_close(struct got_raw_object *obj)
 			return NULL;
 	}
 
+	if (obj->close_cb)
+		obj->close_cb(obj);
+
 	if (obj->f == NULL) {
 		if (obj->fd != -1) {
 			if (munmap(obj->data, obj->hdrlen + obj->size) == -1)
@@ -272,6 +274,85 @@ done:
 	return err;
 }
 
+const struct got_error *
+got_object_read_raw(uint8_t **outbuf, off_t *size, size_t *hdrlen,
+    size_t max_in_mem_size, int outfd, struct got_object_id *expected_id,
+    int infd)
+{
+	const struct got_error *err = NULL;
+	struct got_object *obj;
+	struct got_inflate_checksum csum;
+	uint8_t sha1[SHA1_DIGEST_LENGTH];
+	SHA1_CTX sha1_ctx;
+	size_t len, consumed;
+	FILE *f = NULL;
+
+	*outbuf = NULL;
+	*size = 0;
+	*hdrlen = 0;
+
+	SHA1Init(&sha1_ctx);
+	memset(&csum, 0, sizeof(csum));
+	csum.output_sha1 = &sha1_ctx;
+
+	if (lseek(infd, SEEK_SET, 0) == -1)
+		return got_error_from_errno("lseek");
+
+	err = got_object_read_header(&obj, infd);
+	if (err)
+		return err;
+
+	if (lseek(infd, SEEK_SET, 0) == -1)
+		return got_error_from_errno("lseek");
+
+	if (obj->size + obj->hdrlen <= max_in_mem_size) {
+		err = got_inflate_to_mem_fd(outbuf, &len, &consumed, &csum,
+		    obj->size + obj->hdrlen, infd);
+	} else {
+		int fd;
+		/*
+		 * XXX This uses an extra file descriptor for no good reason.
+		 * We should have got_inflate_fd_to_fd().
+		 */
+		fd = dup(infd);
+		if (fd == -1)
+			return got_error_from_errno("dup");
+		f = fdopen(fd, "r");
+		if (f == NULL) {
+			err = got_error_from_errno("fdopen");
+			abort();
+			close(fd);
+			goto done;
+		}
+		err = got_inflate_to_fd(&len, f, &csum, outfd);
+	}
+	if (err)
+		goto done;
+
+	if (len < obj->hdrlen || len != obj->hdrlen + obj->size) {
+		err = got_error(GOT_ERR_BAD_OBJ_HDR);
+		goto done;
+	}
+
+	SHA1Final(sha1, &sha1_ctx);
+	if (memcmp(expected_id->sha1, sha1, SHA1_DIGEST_LENGTH) != 0) {
+		char buf[SHA1_DIGEST_STRING_LENGTH];
+		err = got_error_fmt(GOT_ERR_OBJ_CSUM,
+		    "checksum failure for object %s",
+		    got_sha1_digest_to_str(expected_id->sha1, buf,
+		    sizeof(buf)));
+		goto done;
+	}
+
+	*size = obj->size;
+	*hdrlen = obj->hdrlen;
+done:
+	got_object_close(obj);
+	if (f && fclose(f) == EOF && err == NULL)
+		err = got_error_from_errno("fclose");
+	return err;
+}
+
 struct got_commit_object *
 got_object_commit_alloc_partial(void)
 {
@@ -654,6 +735,60 @@ done:
 	return err;
 }
 
+const struct got_error *
+got_object_read_commit(struct got_commit_object **commit, int fd,
+    struct got_object_id *expected_id, size_t expected_size)
+{
+	struct got_object *obj = NULL;
+	const struct got_error *err = NULL;
+	size_t len;
+	uint8_t *p;
+	struct got_inflate_checksum csum;
+	SHA1_CTX sha1_ctx;
+	struct got_object_id id;
+
+	SHA1Init(&sha1_ctx);
+	memset(&csum, 0, sizeof(csum));
+	csum.output_sha1 = &sha1_ctx;
+
+	err = got_inflate_to_mem_fd(&p, &len, NULL, &csum, expected_size, fd);
+	if (err)
+		return err;
+
+	SHA1Final(id.sha1, &sha1_ctx);
+	if (memcmp(expected_id->sha1, id.sha1, SHA1_DIGEST_LENGTH) != 0) {
+		char buf[SHA1_DIGEST_STRING_LENGTH];
+		err = got_error_fmt(GOT_ERR_OBJ_CSUM,
+		    "checksum failure for object %s",
+		    got_sha1_digest_to_str(expected_id->sha1, buf,
+		    sizeof(buf)));
+		goto done;
+	}
+
+	err = got_object_parse_header(&obj, p, len);
+	if (err)
+		goto done;
+
+	if (len < obj->hdrlen + obj->size) {
+		err = got_error(GOT_ERR_BAD_OBJ_DATA);
+		goto done;
+	}
+
+	if (obj->type != GOT_OBJ_TYPE_COMMIT) {
+		err = got_error(GOT_ERR_OBJ_TYPE);
+		goto done;
+	}
+
+	/* Skip object header. */
+	len -= obj->hdrlen;
+	err = got_object_parse_commit(commit, p + obj->hdrlen, len);
+done:
+	free(p);
+	if (obj)
+		got_object_close(obj);
+	return err;
+}
+
 void
 got_object_tree_close(struct got_tree_object *tree)
 {
@@ -713,11 +848,11 @@ pte_cmp(const void *pa, const void *pb)
 }
 
 const struct got_error *
-got_object_parse_tree(struct got_parsed_tree_entry **entries, int *nentries,
-    uint8_t *buf, size_t len)
+got_object_parse_tree(struct got_parsed_tree_entry **entries, size_t *nentries,
+    size_t *nentries_alloc, uint8_t *buf, size_t len)
 {
 	const struct got_error *err = NULL;
-	size_t remain = len, totalloc;
+	size_t remain = len;
 	const size_t nalloc = 16;
 	struct got_parsed_tree_entry *pte;
 	int i;
@@ -726,23 +861,18 @@ got_object_parse_tree(struct got_parsed_tree_entry **entries, int *nentries,
 	if (remain == 0)
 		return NULL; /* tree is empty */
 
-	*entries = calloc(nalloc, sizeof(**entries));
-	if (*entries == NULL)
-		return got_error_from_errno("calloc");
-	totalloc = nalloc;
-
 	while (remain > 0) {
 		size_t elen;
 
-		if (*nentries >= totalloc) {
-			pte = recallocarray(*entries, totalloc,
-			    totalloc + nalloc, sizeof(**entries));
+		if (*nentries >= *nentries_alloc) {
+			pte = recallocarray(*entries, *nentries_alloc,
+			    *nentries_alloc + nalloc, sizeof(**entries));
 			if (pte == NULL) {
 				err = got_error_from_errno("recallocarray");
 				goto done;
 			}
 			*entries = pte;
-			totalloc += nalloc;
+			*nentries_alloc += nalloc;
 		}
 
 		pte = &(*entries)[*nentries];
@@ -773,11 +903,57 @@ got_object_parse_tree(struct got_parsed_tree_entry **entries, int *nentries,
 		}
 	}
 done:
-	if (err) {
-		free(*entries);
-		*entries = NULL;
+	if (err)
 		*nentries = 0;
+	return err;
+}
+
+const struct got_error *
+got_object_read_tree(struct got_parsed_tree_entry **entries, size_t *nentries,
+    size_t *nentries_alloc, uint8_t **p, int fd,
+    struct got_object_id *expected_id)
+{
+	const struct got_error *err = NULL;
+	struct got_object *obj = NULL;
+	size_t len;
+	struct got_inflate_checksum csum;
+	SHA1_CTX sha1_ctx;
+	struct got_object_id id;
+
+	SHA1Init(&sha1_ctx);
+	memset(&csum, 0, sizeof(csum));
+	csum.output_sha1 = &sha1_ctx;
+
+	err = got_inflate_to_mem_fd(p, &len, NULL, &csum, 0, fd);
+	if (err)
+		return err;
+
+	SHA1Final(id.sha1, &sha1_ctx);
+	if (memcmp(expected_id->sha1, id.sha1, SHA1_DIGEST_LENGTH) != 0) {
+		char buf[SHA1_DIGEST_STRING_LENGTH];
+		err = got_error_fmt(GOT_ERR_OBJ_CSUM,
+		    "checksum failure for object %s",
+		    got_sha1_digest_to_str(expected_id->sha1, buf,
+		    sizeof(buf)));
+		goto done;
 	}
+
+	err = got_object_parse_header(&obj, *p, len);
+	if (err)
+		goto done;
+
+	if (len < obj->hdrlen + obj->size) {
+		err = got_error(GOT_ERR_BAD_OBJ_DATA);
+		goto done;
+	}
+
+	/* Skip object header. */
+	len -= obj->hdrlen;
+	err = got_object_parse_tree(entries, nentries, nentries_alloc,
+	    *p + obj->hdrlen, len);
+done:
+	if (obj)
+		got_object_close(obj);
 	return err;
 }
 
@@ -975,6 +1151,56 @@ done:
 	return err;
 }
 
+const struct got_error *
+got_object_read_tag(struct got_tag_object **tag, int fd, 
+    struct got_object_id *expected_id, size_t expected_size)
+{
+	const struct got_error *err = NULL;
+	struct got_object *obj = NULL;
+	size_t len;
+	uint8_t *p;
+	struct got_inflate_checksum csum;
+	SHA1_CTX sha1_ctx;
+	struct got_object_id id;
+
+	SHA1Init(&sha1_ctx);
+	memset(&csum, 0, sizeof(csum));
+	csum.output_sha1 = &sha1_ctx;
+
+	err = got_inflate_to_mem_fd(&p, &len, NULL, &csum,
+	    expected_size, fd);
+	if (err)
+		return err;
+
+	SHA1Final(id.sha1, &sha1_ctx);
+	if (memcmp(expected_id->sha1, id.sha1, SHA1_DIGEST_LENGTH) != 0) {
+		char buf[SHA1_DIGEST_STRING_LENGTH];
+		err = got_error_fmt(GOT_ERR_OBJ_CSUM,
+		    "checksum failure for object %s",
+		    got_sha1_digest_to_str(expected_id->sha1, buf,
+		    sizeof(buf)));
+		goto done;
+	}
+
+	err = got_object_parse_header(&obj, p, len);
+	if (err)
+		goto done;
+
+	if (len < obj->hdrlen + obj->size) {
+		err = got_error(GOT_ERR_BAD_OBJ_DATA);
+		goto done;
+	}
+
+	/* Skip object header. */
+	len -= obj->hdrlen;
+	err = got_object_parse_tag(tag, p + obj->hdrlen, len);
+done:
+	free(p);
+	if (obj)
+		got_object_close(obj);
+	return err;
+}
+
 const struct got_error *
 got_read_file_to_mem(uint8_t **outbuf, size_t *outlen, FILE *f)
 {
