@@ -788,7 +788,7 @@ done:
 static const struct got_error *
 pack_stop_privsep_child(struct got_pack *pack)
 {
-	const struct got_error *err = NULL;
+	const struct got_error *err = NULL, *close_err = NULL;
 
 	if (pack->privsep_child == NULL)
 		return NULL;
@@ -796,9 +796,12 @@ pack_stop_privsep_child(struct got_pack *pack)
 	err = got_privsep_send_stop(pack->privsep_child->imsg_fd);
 	if (err)
 		return err;
+	if (close(pack->privsep_child->imsg_fd) == -1)
+		close_err = got_error_from_errno("close");
 	err = got_privsep_wait_for_child(pack->privsep_child->pid);
-	if (close(pack->privsep_child->imsg_fd) == -1 && err == NULL)
-		err = got_error_from_errno("close");
+	if (close_err && err == NULL)
+		err = close_err;
+	imsg_clear(pack->privsep_child->ibuf);
 	free(pack->privsep_child->ibuf);
 	free(pack->privsep_child);
 	pack->privsep_child = NULL;
@@ -857,7 +860,8 @@ got_pack_parse_object_type_and_size(uint8_t *type, uint64_t *size, size_t *len,
 	do {
 		/* We do not support size values which don't fit in 64 bit. */
 		if (i > 9)
-			return got_error(GOT_ERR_NO_SPACE);
+			return got_error_fmt(GOT_ERR_OBJ_TOO_LARGE,
+			    "packfile offset %llu", offset);
 
 		if (pack->map) {
 			if (mapoff + sizeof(sizeN) >= pack->filesize)
@@ -1279,18 +1283,22 @@ got_pack_get_delta_chain_max_size(uint64_t *max_size,
 		    delta->type != GOT_OBJ_TYPE_BLOB &&
 		    delta->type != GOT_OBJ_TYPE_TAG) {
 			const struct got_error *err;
-			uint8_t *delta_buf;
+			uint8_t *delta_buf = NULL;
 			size_t delta_len;
 			int cached = 1;
 
-			got_delta_cache_get(&delta_buf, &delta_len,
-			    pack->delta_cache, delta->data_offset);
+			if (pack->delta_cache) {
+				got_delta_cache_get(&delta_buf, &delta_len,
+				    pack->delta_cache, delta->data_offset);
+			}
 			if (delta_buf == NULL) {
 				cached = 0;
 				err = read_delta_data(&delta_buf, &delta_len,
 				    NULL, delta->data_offset, pack);
 				if (err)
 					return err;
+			}
+			if (pack->delta_cache && !cached) {
 				err = got_delta_cache_add(pack->delta_cache,
 				    delta->data_offset, delta_buf, delta_len);
 				if (err == NULL)
@@ -1334,7 +1342,7 @@ got_pack_dump_delta_chain_to_file(size_t *result_size,
 {
 	const struct got_error *err = NULL;
 	struct got_delta *delta;
-	uint8_t *base_buf = NULL, *accum_buf = NULL, *delta_buf;
+	uint8_t *base_buf = NULL, *accum_buf = NULL;
 	size_t base_bufsz = 0, accum_bufsz = 0, accum_size = 0, delta_len;
 	/* We process small enough files entirely in memory for speed. */
 	const size_t max_bufsize = GOT_DELTA_RESULT_SIZE_CACHED_MAX;
@@ -1353,6 +1361,7 @@ got_pack_dump_delta_chain_to_file(size_t *result_size,
 
 	/* Deltas are ordered in ascending order. */
 	STAILQ_FOREACH(delta, &deltas->entries, entry) {
+		uint8_t *delta_buf = NULL;
 		uint64_t base_size, result_size = 0;
 		int cached = 1;
 		if (n == 0) {
@@ -1419,14 +1428,18 @@ got_pack_dump_delta_chain_to_file(size_t *result_size,
 			continue;
 		}
 
-		got_delta_cache_get(&delta_buf, &delta_len,
-		    pack->delta_cache, delta->data_offset);
+		if (pack->delta_cache) {
+			got_delta_cache_get(&delta_buf, &delta_len,
+			    pack->delta_cache, delta->data_offset);
+		}
 		if (delta_buf == NULL) {
 			cached = 0;
 			err = read_delta_data(&delta_buf, &delta_len, NULL,
 			    delta->data_offset, pack);
 			if (err)
 				goto done;
+		}
+		if (pack->delta_cache && !cached) {
 			err = got_delta_cache_add(pack->delta_cache,
 			    delta->data_offset, delta_buf, delta_len);
 			if (err == NULL)
@@ -1536,7 +1549,7 @@ got_pack_dump_delta_chain_to_mem(uint8_t **outbuf, size_t *outlen,
 {
 	const struct got_error *err = NULL;
 	struct got_delta *delta;
-	uint8_t *base_buf = NULL, *accum_buf = NULL, *delta_buf;
+	uint8_t *base_buf = NULL, *accum_buf = NULL;
 	size_t base_bufsz = 0, accum_bufsz = 0, accum_size = 0, delta_len;
 	uint64_t max_size = 0;
 	int n = 0;
@@ -1549,6 +1562,7 @@ got_pack_dump_delta_chain_to_mem(uint8_t **outbuf, size_t *outlen,
 
 	/* Deltas are ordered in ascending order. */
 	STAILQ_FOREACH(delta, &deltas->entries, entry) {
+		uint8_t *delta_buf = NULL;
 		uint64_t base_size, result_size = 0;
 		int cached = 1;
 		if (n == 0) {
@@ -1593,14 +1607,18 @@ got_pack_dump_delta_chain_to_mem(uint8_t **outbuf, size_t *outlen,
 			continue;
 		}
 
-		got_delta_cache_get(&delta_buf, &delta_len,
-		    pack->delta_cache, delta->data_offset);
+		if (pack->delta_cache) {
+			got_delta_cache_get(&delta_buf, &delta_len,
+			    pack->delta_cache, delta->data_offset);
+		}
 		if (delta_buf == NULL) {
 			cached = 0;
 			err = read_delta_data(&delta_buf, &delta_len, NULL,
 			    delta->data_offset, pack);
 			if (err)
 				goto done;
+		}
+		if (pack->delta_cache && !cached) {
 			err = got_delta_cache_add(pack->delta_cache,
 			    delta->data_offset, delta_buf, delta_len);
 			if (err == NULL)
