@@ -19,7 +19,6 @@
 #include <sys/stat.h>
 
 #include <errno.h>
-#include <ctype.h>
 #include <dirent.h>
 #include <limits.h>
 #include <stdio.h>
@@ -268,57 +267,6 @@ get_refs_dir_path(struct got_repository *repo, const char *refname)
 	return got_repo_get_path_refs(repo);
 }
 
-int
-got_ref_name_is_valid(const char *name)
-{
-	const char *s, *seg;
-	const char forbidden[] = { ' ', '~', '^', ':', '?', '*', '[' , '\\' };
-	const char *forbidden_seq[] = { "//", "..", "@{" };
-	const char *lfs = GOT_LOCKFILE_SUFFIX;
-	const size_t lfs_len = sizeof(GOT_LOCKFILE_SUFFIX) - 1;
-	size_t i;
-
-	if (name[0] == '@' && name[1] == '\0')
-		return 0;
-
-	s = name;
-	seg = s;
-	if (seg[0] == '\0' || seg[0] == '.' || seg[0] == '/')
-		return 0;
-	while (*s) {
-		for (i = 0; i < nitems(forbidden); i++) {
-			if (*s == forbidden[i])
-				return 0;
-		}
-		for (i = 0; i < nitems(forbidden_seq); i++) {
-			if (s[0] == forbidden_seq[i][0] &&
-			    s[1] == forbidden_seq[i][1])
-				return 0;
-		}
-		if (iscntrl((unsigned char)s[0]))
-			return 0;
-		if (s[0] == '.' && s[1] == '\0')
-			return 0;
-		if (*s == '/') {
-			const char *nextseg = s + 1;
-			if (nextseg[0] == '\0' || nextseg[0] == '.' ||
-			    nextseg[0] == '/')
-				return 0;
-			if (seg <= s - lfs_len &&
-			    strncmp(s - lfs_len, lfs, lfs_len) == 0)
-				return 0;
-			seg = nextseg;
-		}
-		s++;
-	}
-
-	if (seg <= s - lfs_len &&
-	    strncmp(s - lfs_len, lfs, lfs_len) == 0)
-		return 0;
-
-	return 1;
-}
-
 const struct got_error *
 got_ref_alloc(struct got_reference **ref, const char *name,
     struct got_object_id *id)
@@ -1320,7 +1268,7 @@ done:
 	free(path_refs);
 	free(path);
 	if (tmppath) {
-		if (unlink(tmppath) != 0 && err == NULL)
+		if (unlink(tmppath) == -1 && err == NULL)
 			err = got_error_from_errno2("unlink", tmppath);
 		free(tmppath);
 	}
@@ -1454,6 +1402,8 @@ delete_packed_ref(struct got_reference *delref, struct got_repository *repo)
 			    packed_refs_path);
 			goto done;
 		}
+		free(tmppath);
+		tmppath = NULL;
 	}
 done:
 	if (delref->lf == NULL && lf)
@@ -1462,11 +1412,10 @@ done:
 		if (fclose(f) == EOF && err == NULL)
 			err = got_error_from_errno("fclose");
 	}
-	if (tmpf) {
-		unlink(tmppath);
-		if (fclose(tmpf) == EOF && err == NULL)
-			err = got_error_from_errno("fclose");
-	}
+	if (tmppath && unlink(tmppath) == -1 && err == NULL)
+		err = got_error_from_errno2("unlink", tmppath);
+	if (tmpf && fclose(tmpf) == EOF && err == NULL)
+		err = got_error_from_errno("fclose");
 	free(tmppath);
 	free(packed_refs_path);
 	free(line);
@@ -1501,7 +1450,7 @@ delete_loose_ref(struct got_reference *ref, struct got_repository *repo)
 
 	/* XXX: check if old content matches our expectations? */
 
-	if (unlink(path) != 0)
+	if (unlink(path) == -1)
 		err = got_error_from_errno2("unlink", path);
 done:
 	if (ref->lf == NULL && lf)
