@@ -48,6 +48,7 @@
 #include "got_opentemp.h"
 
 #include "got_lib_delta.h"
+#include "got_lib_delta_cache.h"
 #include "got_lib_inflate.h"
 #include "got_lib_object.h"
 #include "got_lib_object_parse.h"
@@ -243,31 +244,43 @@ done:
 
 }
 
-const struct got_error *
-got_repo_pack_fds_open(int **pack_fds)
+static const struct got_error *
+close_tempfiles(int *fds, size_t nfds)
 {
 	const struct got_error *err = NULL;
 	int i;
 
-	*pack_fds = calloc(GOT_PACK_NUM_TEMPFILES, sizeof(**pack_fds));
-	if (*pack_fds == NULL)
+	for (i = 0; i < nfds; i++) {
+		if (fds[i] == -1)
+			continue;
+		if (close(fds[i]) == -1) {
+			err = got_error_from_errno("close");
+			break;
+		}
+	}
+	free(fds);
+	return err;
+}
+
+static const struct got_error *
+open_tempfiles(int **fds, size_t nfds)
+{
+	const struct got_error *err = NULL;
+	int i;
+
+	*fds = calloc(nfds, sizeof(**fds));
+	if (*fds == NULL)
 		return got_error_from_errno("calloc");
 
-	/*
-	 * got_repo_pack_fds_close will try to close all of the
-	 * GOT_PACK_NUM_TEMPFILES fds, even the ones that didn't manage to get
-	 * a value from got_opentempfd(), which would result in a close(0) if
-	 * we do not initialize to -1 here.
-	 */
-	for (i = 0; i < GOT_PACK_NUM_TEMPFILES; i++)
-		(*pack_fds)[i] = -1;
+	for (i = 0; i < nfds; i++)
+		(*fds)[i] = -1;
 
-	for (i = 0; i < GOT_PACK_NUM_TEMPFILES; i++) {
-		(*pack_fds)[i] = got_opentempfd();
-		if ((*pack_fds)[i] == -1) {
+	for (i = 0; i < nfds; i++) {
+		(*fds)[i] = got_opentempfd();
+		if ((*fds)[i] == -1) {
 			err = got_error_from_errno("got_opentempfd");
-			got_repo_pack_fds_close(*pack_fds);
-			*pack_fds = NULL;
+			close_tempfiles(*fds, nfds);
+			*fds = NULL;
 			return err;
 		}
 	}
@@ -275,22 +288,67 @@ got_repo_pack_fds_open(int **pack_fds)
 	return NULL;
 }
 
+const struct got_error *
+got_repo_pack_fds_open(int **pack_fds)
+{
+	return open_tempfiles(pack_fds, GOT_PACK_NUM_TEMPFILES);
+}
+
 const struct got_error *
 got_repo_pack_fds_close(int *pack_fds)
 {
-	const struct got_error *err = NULL;
+	return close_tempfiles(pack_fds, GOT_PACK_NUM_TEMPFILES);
+}
+
+const struct got_error *
+got_repo_temp_fds_open(int **temp_fds) 
+{
+	return open_tempfiles(temp_fds, GOT_REPO_NUM_TEMPFILES);
+}
+
+void
+got_repo_temp_fds_set(struct got_repository *repo, int *temp_fds) 
+{
 	int i;
 
-	for (i = 0; i < GOT_PACK_NUM_TEMPFILES; i++) {
-		if (pack_fds[i] == -1)
+	for (i = 0; i < GOT_REPO_NUM_TEMPFILES; i++)
+		repo->tempfiles[i] = temp_fds[i];
+}
+
+const struct got_error *
+got_repo_temp_fds_get(int *fd, int *idx, struct got_repository *repo)
+{
+	int i;
+
+	*fd = -1;
+	*idx = -1;
+
+	for (i = 0; i < nitems(repo->tempfiles); i++) {
+		if (repo->tempfile_use_mask & (1 << i))
 			continue;
-		if (close(pack_fds[i]) == -1) {
-			err = got_error_from_errno("close");
-			break;
+		if (repo->tempfiles[i] != -1) {
+			if (ftruncate(repo->tempfiles[i], 0L) == -1)
+				return got_error_from_errno("ftruncate");
+			*fd = repo->tempfiles[i];
+			*idx = i;
+			repo->tempfile_use_mask |= (1 << i);
+			return NULL;
 		}
 	}
-	free(pack_fds);
-	return err;
+
+	return got_error(GOT_ERR_REPO_TEMPFILE);
+}
+
+void
+got_repo_temp_fds_put(int idx, struct got_repository *repo)
+{
+	repo->tempfile_use_mask &= ~(1 << idx);
+}
+
+const struct got_error *
+got_repo_temp_fds_close(int *temp_fds)
+{
+	return close_tempfiles(temp_fds, GOT_REPO_NUM_TEMPFILES);
 }
 
 const struct got_error *
@@ -487,159 +545,6 @@ done:
 	return err;
 }
 
-static const struct got_error *
-parse_gitconfig_file(int *gitconfig_repository_format_version,
-    char **gitconfig_author_name, char **gitconfig_author_email,
-    struct got_remote_repo **remotes, int *nremotes,
-    char **gitconfig_owner, char ***extensions, int *nextensions,
-    const char *gitconfig_path)
-{
-	const struct got_error *err = NULL, *child_err = NULL;
-	int fd = -1;
-	int imsg_fds[2] = { -1, -1 };
-	pid_t pid;
-	struct imsgbuf *ibuf;
-
-	*gitconfig_repository_format_version = 0;
-	if (extensions)
-		*extensions = NULL;
-	if (nextensions)
-		*nextensions = 0;
-	*gitconfig_author_name = NULL;
-	*gitconfig_author_email = NULL;
-	if (remotes)
-		*remotes = NULL;
-	if (nremotes)
-		*nremotes = 0;
-	if (gitconfig_owner)
-		*gitconfig_owner = NULL;
-
-	fd = open(gitconfig_path, O_RDONLY | O_CLOEXEC);
-	if (fd == -1) {
-		if (errno == ENOENT)
-			return NULL;
-		return got_error_from_errno2("open", gitconfig_path);
-	}
-
-	ibuf = calloc(1, sizeof(*ibuf));
-	if (ibuf == NULL) {
-		err = got_error_from_errno("calloc");
-		goto done;
-	}
-
-	if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC, imsg_fds) == -1) {
-		err = got_error_from_errno("socketpair");
-		goto done;
-	}
-
-	pid = fork();
-	if (pid == -1) {
-		err = got_error_from_errno("fork");
-		goto done;
-	} else if (pid == 0) {
-		got_privsep_exec_child(imsg_fds, GOT_PATH_PROG_READ_GITCONFIG,
-		    gitconfig_path);
-		/* not reached */
-	}
-
-	if (close(imsg_fds[1]) == -1) {
-		err = got_error_from_errno("close");
-		goto done;
-	}
-	imsg_fds[1] = -1;
-	imsg_init(ibuf, imsg_fds[0]);
-
-	err = got_privsep_send_gitconfig_parse_req(ibuf, fd);
-	if (err)
-		goto done;
-	fd = -1;
-
-	err = got_privsep_send_gitconfig_repository_format_version_req(ibuf);
-	if (err)
-		goto done;
-
-	err = got_privsep_recv_gitconfig_int(
-	    gitconfig_repository_format_version, ibuf);
-	if (err)
-		goto done;
-
-	if (extensions && nextensions) {
-		err = got_privsep_send_gitconfig_repository_extensions_req(
-		    ibuf);
-		if (err)
-			goto done;
-		err = got_privsep_recv_gitconfig_int(nextensions, ibuf);
-		if (err)
-			goto done;
-		if (*nextensions > 0) {
-			int i;
-			*extensions = calloc(*nextensions, sizeof(char *));
-			if (*extensions == NULL) {
-				err = got_error_from_errno("calloc");
-				goto done;
-			}
-			for (i = 0; i < *nextensions; i++) {
-				char *ext;
-				err = got_privsep_recv_gitconfig_str(&ext,
-				    ibuf);
-				if (err)
-					goto done;
-				(*extensions)[i] = ext;
-			}
-		}
-	}
-
-	err = got_privsep_send_gitconfig_author_name_req(ibuf);
-	if (err)
-		goto done;
-
-	err = got_privsep_recv_gitconfig_str(gitconfig_author_name, ibuf);
-	if (err)
-		goto done;
-
-	err = got_privsep_send_gitconfig_author_email_req(ibuf);
-	if (err)
-		goto done;
-
-	err = got_privsep_recv_gitconfig_str(gitconfig_author_email, ibuf);
-	if (err)
-		goto done;
-
-	if (remotes && nremotes) {
-		err = got_privsep_send_gitconfig_remotes_req(ibuf);
-		if (err)
-			goto done;
-
-		err = got_privsep_recv_gitconfig_remotes(remotes,
-		    nremotes, ibuf);
-		if (err)
-			goto done;
-	}
-
-	if (gitconfig_owner) {
-		err = got_privsep_send_gitconfig_owner_req(ibuf);
-		if (err)
-			goto done;
-		err = got_privsep_recv_gitconfig_str(gitconfig_owner, ibuf);
-		if (err)
-			goto done;
-	}
-
-	err = got_privsep_send_stop(imsg_fds[0]);
-	child_err = got_privsep_wait_for_child(pid);
-	if (child_err && err == NULL)
-		err = child_err;
-done:
-	if (imsg_fds[0] != -1 && close(imsg_fds[0]) == -1 && err == NULL)
-		err = got_error_from_errno("close");
-	if (imsg_fds[1] != -1 && close(imsg_fds[1]) == -1 && err == NULL)
-		err = got_error_from_errno("close");
-	if (fd != -1 && close(fd) == -1 && err == NULL)
-		err = got_error_from_errno2("close", gitconfig_path);
-	free(ibuf);
-	return err;
-}
-
 static const struct got_error *
 read_gitconfig(struct got_repository *repo, const char *global_gitconfig_path)
 {
@@ -649,7 +554,7 @@ read_gitconfig(struct got_repository *repo, const char *global_gitconfig_path)
 	if (global_gitconfig_path) {
 		/* Read settings from ~/.gitconfig. */
 		int dummy_repo_version;
-		err = parse_gitconfig_file(&dummy_repo_version,
+		err = got_repo_read_gitconfig(&dummy_repo_version,
 		    &repo->global_gitconfig_author_name,
 		    &repo->global_gitconfig_author_email,
 		    NULL, NULL, NULL, NULL, NULL, global_gitconfig_path);
@@ -662,7 +567,8 @@ read_gitconfig(struct got_repository *repo, const char *global_gitconfig_path)
 	if (repo_gitconfig_path == NULL)
 		return got_error_from_errno("got_repo_get_path_gitconfig");
 
-	err = parse_gitconfig_file(&repo->gitconfig_repository_format_version,
+	err = got_repo_read_gitconfig(
+	    &repo->gitconfig_repository_format_version,
 	    &repo->gitconfig_author_name, &repo->gitconfig_author_email,
 	    &repo->gitconfig_remotes, &repo->ngitconfig_remotes,
 	    &repo->gitconfig_owner, &repo->extensions, &repo->nextensions,
@@ -772,7 +678,7 @@ got_repo_open(struct got_repository **repop, const char *path,
 	if (repo->pack_cache_size > rl.rlim_cur / 8)
 		repo->pack_cache_size = rl.rlim_cur / 8;
 	for (i = 0; i < nitems(repo->packs); i++) {
-		if (i < repo->pack_cache_size) {
+		if (pack_fds != NULL && i < repo->pack_cache_size) {
 			repo->packs[i].basefd = pack_fds[j++];
 			repo->packs[i].accumfd = pack_fds[j++];
 		} else {
@@ -780,6 +686,8 @@ got_repo_open(struct got_repository **repop, const char *path,
 			repo->packs[i].accumfd = -1;
 		}
 	}
+	for (i = 0; i < nitems(repo->tempfiles); i++)
+		repo->tempfiles[i] = -1;
 	repo->pinned_pack = -1;
 	repo->pinned_packidx = -1;
 	repo->pinned_pid = 0;
@@ -1181,6 +1089,46 @@ add_packidx_bloom_filter(struct got_repository *repo,
 	return NULL;
 }
 
+static void
+purge_packidx_paths(struct got_pathlist_head *packidx_paths)
+{
+	struct got_pathlist_entry *pe;
+
+	while (!TAILQ_EMPTY(packidx_paths)) {
+		pe = TAILQ_FIRST(packidx_paths);
+		TAILQ_REMOVE(packidx_paths, pe, entry);
+		free((char *)pe->path);
+		free(pe);
+	}
+}
+
+static const struct got_error *
+refresh_packidx_paths(struct got_repository *repo)
+{
+	const struct got_error *err = NULL;
+	char *objects_pack_dir = NULL;
+	struct stat sb;
+
+	objects_pack_dir = got_repo_get_path_objects_pack(repo);
+	if (objects_pack_dir == NULL)
+		return got_error_from_errno("got_repo_get_path_objects_pack");
+
+	if (stat(objects_pack_dir, &sb) == -1) {
+		if (errno != ENOENT) {
+			err = got_error_from_errno2("stat", objects_pack_dir);
+			goto done;
+		}
+	} else if (sb.st_mtime != repo->pack_path_mtime) {
+		purge_packidx_paths(&repo->packidx_paths);
+		err = got_repo_list_packidx(&repo->packidx_paths, repo);
+		if (err)
+			goto done;
+	}
+done:
+	free(objects_pack_dir);
+	return err;
+}
+
 const struct got_error *
 got_repo_search_packidx(struct got_packidx **packidx, int *idx,
     struct got_repository *repo, struct got_object_id *id)
@@ -1219,6 +1167,10 @@ got_repo_search_packidx(struct got_packidx **packidx, int *idx,
 	}
 	/* No luck. Search the filesystem. */
 
+	err = refresh_packidx_paths(repo);
+	if (err)
+		return err;
+
 	TAILQ_FOREACH(pe, &repo->packidx_paths, entry) {
 		const char *path_packidx = pe->path;
 		int is_cached = 0;
@@ -1467,6 +1419,10 @@ got_repo_cache_pack(struct got_pack **packp, struct got_repository *repo,
 
 	pack->privsep_child = NULL;
 
+	err = got_delta_cache_alloc(&pack->delta_cache);
+	if (err)
+		goto done;
+
 #ifndef GOT_PACK_NO_MMAP
 	pack->map = mmap(NULL, pack->filesize, PROT_READ, MAP_PRIVATE,
 	    pack->fd, 0);
@@ -1480,10 +1436,8 @@ got_repo_cache_pack(struct got_pack **packp, struct got_repository *repo,
 #endif
 done:
 	if (err) {
-		if (pack) {
-			free(pack->path_packfile);
-			memset(pack, 0, sizeof(*pack));
-		}
+		if (pack)
+			got_pack_close(pack);
 	} else if (packp)
 		*packp = pack;
 	return err;
@@ -1613,46 +1567,19 @@ got_repo_init(const char *repo_path, const char *head_name)
 	return NULL;
 }
 
-static void
-purge_packidx_paths(struct got_pathlist_head *packidx_paths)
-{
-	struct got_pathlist_entry *pe;
-
-	while (!TAILQ_EMPTY(packidx_paths)) {
-		pe = TAILQ_FIRST(packidx_paths);
-		TAILQ_REMOVE(packidx_paths, pe, entry);
-		free((char *)pe->path);
-		free(pe);
-	}
-}
-
 static const struct got_error *
 match_packed_object(struct got_object_id **unique_id,
     struct got_repository *repo, const char *id_str_prefix, int obj_type)
 {
 	const struct got_error *err = NULL;
-	char *objects_pack_dir = NULL;
 	struct got_object_id_queue matched_ids;
 	struct got_pathlist_entry *pe;
-	struct stat sb;
 
 	STAILQ_INIT(&matched_ids);
 
-	objects_pack_dir = got_repo_get_path_objects_pack(repo);
-	if (objects_pack_dir == NULL)
-		return got_error_from_errno("got_repo_get_path_objects_pack");
-
-	if (stat(objects_pack_dir, &sb) == -1) {
-		if (errno != ENOENT) {
-			err = got_error_from_errno2("stat", objects_pack_dir);
-			goto done;
-		}
-	} else if (sb.st_mtime != repo->pack_path_mtime) {
-		purge_packidx_paths(&repo->packidx_paths);
-		err = got_repo_list_packidx(&repo->packidx_paths, repo);
-		if (err)
-			goto done;
-	}
+	err = refresh_packidx_paths(repo);
+	if (err)
+		return err;
 
 	TAILQ_FOREACH(pe, &repo->packidx_paths, entry) {
 		const char *path_packidx = pe->path;
@@ -1700,7 +1627,6 @@ match_packed_object(struct got_object_id **unique_id,
 		}
 	}
 done:
-	free(objects_pack_dir);
 	got_object_id_queue_free(&matched_ids);
 	if (err) {
 		free(*unique_id);
