@@ -47,10 +47,10 @@
 #include "got_lib_pack.h"
 #include "got_lib_privsep.h"
 #include "got_lib_repository.h"
+#include "got_lib_ratelimit.h"
 #include "got_lib_pack_create.h"
 #include "got_lib_sha1.h"
 #include "got_lib_lockfile.h"
-#include "got_lib_ratelimit.h"
 
 #ifndef nitems
 #define nitems(_a)	(sizeof((_a)) / sizeof((_a)[0]))
@@ -150,10 +150,14 @@ got_repo_pack_objects(FILE **packfile, struct got_object_id **pack_hash,
 	int nours = 0, ntheirs = 0, packfd = -1, i;
 	char *tmpfile_path = NULL, *path = NULL, *packfile_path = NULL;
 	char *sha1_str = NULL;
+	FILE *delta_cache = NULL;
+	struct got_ratelimit rl;
 
 	*packfile = NULL;
 	*pack_hash = NULL;
 
+	got_ratelimit_init(&rl, 0, 500);
+
 	if (asprintf(&path, "%s/%s/packing.pack",
 	    got_repo_get_path_git_dir(repo), GOT_OBJECTS_PACK_DIR) == -1) {
 		err = got_error_from_errno("asprintf");
@@ -168,12 +172,11 @@ got_repo_pack_objects(FILE **packfile, struct got_object_id **pack_hash,
 		goto done;
 	}
 
-	*packfile = fdopen(packfd, "w");
-	if (*packfile == NULL) {
-		err = got_error_from_errno2("fdopen", tmpfile_path);
+	delta_cache = got_opentemp();
+	if (delta_cache == NULL) {
+		err = got_error_from_errno("got_opentemp");
 		goto done;
 	}
-	packfd = -1;
 
 	err = get_reflist_object_ids(&ours, &nours,
 	    (1 << GOT_OBJ_TYPE_COMMIT) | (1 << GOT_OBJ_TYPE_TAG),
@@ -201,9 +204,9 @@ got_repo_pack_objects(FILE **packfile, struct got_object_id **pack_hash,
 		goto done;
 	}
 
-	err = got_pack_create((*pack_hash)->sha1, *packfile, theirs, ntheirs,
-	    ours, nours, repo, loose_obj_only, 0, progress_cb, progress_arg,
-	    cancel_cb, cancel_arg);
+	err = got_pack_create((*pack_hash)->sha1, packfd, delta_cache,
+	    theirs, ntheirs, ours, nours, repo, loose_obj_only, 0,
+	    progress_cb, progress_arg, &rl, cancel_cb, cancel_arg);
 	if (err)
 		goto done;
 
@@ -217,12 +220,8 @@ got_repo_pack_objects(FILE **packfile, struct got_object_id **pack_hash,
 		goto done;
 	}
 
-	if (fflush(*packfile) == -1) {
-		err = got_error_from_errno("fflush");
-		goto done;
-	}
-	if (fseek(*packfile, 0L, SEEK_SET) == -1) {
-		err = got_error_from_errno("fseek");
+	if (lseek(packfd, 0L, SEEK_SET) == -1) {
+		err = got_error_from_errno("lseek");
 		goto done;
 	}
 	if (rename(tmpfile_path, packfile_path) == -1) {
@@ -232,6 +231,13 @@ got_repo_pack_objects(FILE **packfile, struct got_object_id **pack_hash,
 	}
 	free(tmpfile_path);
 	tmpfile_path = NULL;
+
+	*packfile = fdopen(packfd, "w");
+	if (*packfile == NULL) {
+		err = got_error_from_errno2("fdopen", tmpfile_path);
+		goto done;
+	}
+	packfd = -1;
 done:
 	for (i = 0; i < nours; i++)
 		free(ours[i]);
@@ -241,6 +247,8 @@ done:
 	free(theirs);
 	if (packfd != -1 && close(packfd) == -1 && err == NULL)
 		err = got_error_from_errno2("close", packfile_path);
+	if (delta_cache && fclose(delta_cache) == EOF && err == NULL)
+		err = got_error_from_errno("fclose");
 	if (tmpfile_path && unlink(tmpfile_path) == -1 && err == NULL)
 		err = got_error_from_errno2("unlink", tmpfile_path);
 	free(tmpfile_path);
